/* 
 * ASUS K501UQ - OpenCore hotpatches
 * AsusSMC additions for keyboard backlight and Fn keys.
 *
 * This patch adds fake ambient light sensor, enables 
 * keyboard backlight and increases keyboard backlight levels
 * to 15, enables several Fn keys: F2, F5, F6, F8.
 *
 * Important: Check what EC register is accessed in original
 * keyboard backlight method SLKB in DSDT. It looks like
 *  _SB.PCI0.LPCB.EC0.WRAM (0xF8B1, Local0)
 * Custom SKBV method should use this register. Double-check that.
 *
 * Required patches:
 * 15-levels keyboard backlight, PWKB => XWKB in DSDT
 *     0850574B42 => 0858574B42
 * Fn keys, _Q0B => XQ0B, etc., all in DSDT
 *     5F513042 00A01292 => 58513042 00A01292
 *     5F513045 00A00E95 => 58513045 00A00E95
 *     5F513046 00A00E95 => 58513046 00A00E95
 *     5F513131 08085F54 => 58513131 08085F54
 */

DefinitionBlock ("", "SSDT", 2, "hack", "ASMC", 0x00000000)
{
    External (_SB.ATKD, DeviceObj)
    External (_SB.KBLV, FieldUnitObj)
    External (_SB.PCI0.LPCB.EC0, DeviceObj)
    External (_SB.PCI0.LPCB.EC0.WRAM, MethodObj)
    External (_SB.ATKP, IntObj)
    External (_SB.ATKD.IANE, MethodObj)
    External (_SB.PCI0.LPCB.EC0.XQ0B, MethodObj)
    External (_SB.PCI0.LPCB.EC0.XQ0E, MethodObj)
    External (_SB.PCI0.LPCB.EC0.XQ0F, MethodObj)
    External (_SB.PCI0.LPCB.EC0.XQ11, MethodObj)
    
    // Fake ambent light sensor
    Scope (_SB) {
        Device (ALS0)
        {
            Name (_HID, "ACPI0008" /* Ambient Light Sensor Device */)  // _HID: Hardware ID
            Name (_CID, "smc-als")  // _CID: Compatible ID
            Name (_ALI, 0x012C)  // _ALI: Ambient Light Illuminance
            Name (_ALR, Package (0x01)  // _ALR: Ambient Light Response
            {
                Package (0x02)
                {
                    0x64, 
                    0x012C
                }
            })
            Method (_STA, 0, NotSerialized)  // _STA: Status
            {
                If (_OSI ("Darwin"))
                {
                    Return (0x0F)
                }
                Else
                {
                    Return (Zero)
                }
            }
        }
    }
    Scope (_SB.ATKD) 
    {
    
        // 16 keyboard backlight levels instead of 4
        // Rename original PWKB
        Name (PWKB, Buffer (0x10)
            {
                /* 0000 */  0x00, 0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77,  
                /* 0008 */  0x88, 0x99, 0xAA, 0xBB, 0xCC, 0xDD, 0xEE, 0xFF   
            })
    
        // Adding ALS methods
        Method (ALSS, 0, NotSerialized)
            {
                Return (^^ALS0._ALI) 
            }

        Method (ALSC, 1, NotSerialized)
            {
            }

        // Setting keyboard backlight
        // Ensure that EC write register is exactly same as in original SLKB method
        Method (SKBV, 1, NotSerialized)
            {
                Divide (Arg0, 0x10, , KBLV) 
                Store (DerefOf (Index (PWKB, KBLV)), Local0)
                ^^PCI0.LPCB.EC0.WRAM (0xF8B1, Local0)
                Return (Arg0)
            }
    }
    Scope (_SB.PCI0.LPCB.EC0) 
    {
// Fn keys patches
// If Darwin, use short method, else call original
        Method (_Q0B, 0, NotSerialized)  // _Qxx: EC Query, xx=0x00-0xFF
        {
            If (_OSI ("Darwin"))
                {
                If (ATKP)
                    { ^^^^ATKD.IANE (0x7D) }
                } Else { XQ0B () }
                
        }
        Method (_Q0E, 0, NotSerialized)  // _Qxx: EC Query, xx=0x00-0xFF
        {
            If (_OSI ("Darwin"))
                {
                If (ATKP)
                    { ^^^^ATKD.IANE (0x20) }
                } Else { XQ0E () }
        }
        Method (_Q0F, 0, NotSerialized)  // _Qxx: EC Query, xx=0x00-0xFF
        {
            If (_OSI ("Darwin"))
                {
                If (ATKP)
                    { ^^^^ATKD.IANE (0x10) }
                } Else { XQ0F () }
        }
        Method (_Q11, 0, Serialized)  // _Qxx: EC Query, xx=0x00-0xFF
        {
            If (_OSI ("Darwin"))
                {
                If (ATKP)
                    { ^^^^ATKD.IANE (0x61) }
                } Else { XQ11 () }
        }
    }
}